@@ -109,35 +109,35 @@ class BucketController(Controller):
             'limit': max_keys + 1,
         }
         if 'prefix' in req.params:
-            query['prefix'] = req.params['prefix']
+            query['prefix'] = swob.wsgi_to_str(req.params['prefix'])
         if 'delimiter' in req.params:
-            query['delimiter'] = req.params['delimiter']
+            query['delimiter'] = swob.wsgi_to_str(req.params['delimiter'])
         fetch_owner = False
         if 'versions' in req.params:
-            query['versions'] = req.params['versions']
+            query['versions'] = swob.wsgi_to_str(req.params['versions'])
             listing_type = 'object-versions'
+            version_marker = swob.wsgi_to_str(req.params.get(
+                'version-id-marker'))
             if 'key-marker' in req.params:
-                query['marker'] = req.params['key-marker']
-                version_marker = req.params.get('version-id-marker')
+                query['marker'] = swob.wsgi_to_str(req.params['key-marker'])
                 if version_marker is not None:
                     if version_marker != 'null':
                         try:
                             Timestamp(version_marker)
                         except ValueError:
                             raise InvalidArgument(
-                                'version-id-marker',
-                                req.params['version-id-marker'],
+                                'version-id-marker', version_marker,
                                 'Invalid version id specified')
                     query['version_marker'] = version_marker
-            elif 'version-id-marker' in req.params:
+            elif version_marker is not None:
                 err_msg = ('A version-id marker cannot be specified without '
                            'a key marker.')
                 raise InvalidArgument('version-id-marker',
-                                      req.params['version-id-marker'], err_msg)
+                                      version_marker, err_msg)
         elif int(req.params.get('list-type', '1')) == 2:
             listing_type = 'version-2'
             if 'start-after' in req.params:
-                query['marker'] = req.params['start-after']
+                query['marker'] = swob.wsgi_to_str(req.params['start-after'])
             # continuation-token overrides start-after
             if 'continuation-token' in req.params:
                 decoded = b64decode(req.params['continuation-token'])
@@ -149,17 +149,24 @@ class BucketController(Controller):
         else:
             listing_type = 'version-1'
             if 'marker' in req.params:
-                query['marker'] = req.params['marker']
+                query['marker'] = swob.wsgi_to_str(req.params['marker'])
 
         return encoding_type, query, listing_type, fetch_owner
 
-    def _build_versions_result(self, req, objects, is_truncated):
+    def _build_versions_result(self, req, objects, encoding_type,
+                               tag_max_keys, is_truncated):
         elem = Element('ListVersionsResult')
         SubElement(elem, 'Name').text = req.container_name
-        SubElement(elem, 'Prefix').text = req.params.get('prefix')
-        SubElement(elem, 'KeyMarker').text = req.params.get('key-marker')
-        SubElement(elem, 'VersionIdMarker').text = req.params.get(
-            'version-id-marker')
+        prefix = swob.wsgi_to_str(req.params.get('prefix'))
+        if prefix and encoding_type == 'url':
+            prefix = quote(prefix)
+        SubElement(elem, 'Prefix').text = prefix
+        key_marker = swob.wsgi_to_str(req.params.get('key-marker'))
+        if key_marker and encoding_type == 'url':
+            key_marker = quote(key_marker)
+        SubElement(elem, 'KeyMarker').text = key_marker
+        SubElement(elem, 'VersionIdMarker').text = swob.wsgi_to_str(
+            req.params.get('version-id-marker'))
         if is_truncated:
             if 'name' in objects[-1]:
                 SubElement(elem, 'NextKeyMarker').text = \
@@ -170,18 +177,34 @@ class BucketController(Controller):
                 SubElement(elem, 'NextKeyMarker').text = \
                     objects[-1]['subdir']
                 SubElement(elem, 'NextVersionIdMarker').text = 'null'
+        SubElement(elem, 'MaxKeys').text = str(tag_max_keys)
+        delimiter = swob.wsgi_to_str(req.params.get('delimiter'))
+        if delimiter is not None:
+            if encoding_type == 'url':
+                delimiter = quote(delimiter)
+            SubElement(elem, 'Delimiter').text = delimiter
+        if encoding_type == 'url':
+            SubElement(elem, 'EncodingType').text = encoding_type
+        SubElement(elem, 'IsTruncated').text = \
+            'true' if is_truncated else 'false'
         return elem
 
-    def _build_base_listing_element(self, req):
+    def _build_base_listing_element(self, req, encoding_type):
         elem = Element('ListBucketResult')
         SubElement(elem, 'Name').text = req.container_name
-        SubElement(elem, 'Prefix').text = req.params.get('prefix')
+        prefix = swob.wsgi_to_str(req.params.get('prefix'))
+        if prefix and encoding_type == 'url':
+            prefix = quote(prefix)
+        SubElement(elem, 'Prefix').text = prefix
         return elem
 
     def _build_list_bucket_result_type_one(self, req, objects, encoding_type,
-                                           is_truncated):
-        elem = self._build_base_listing_element(req)
-        SubElement(elem, 'Marker').text = req.params.get('marker')
+                                           tag_max_keys, is_truncated):
+        elem = self._build_base_listing_element(req, encoding_type)
+        marker = swob.wsgi_to_str(req.params.get('marker'))
+        if marker and encoding_type == 'url':
+            marker = quote(marker)
+        SubElement(elem, 'Marker').text = marker
         if is_truncated and 'delimiter' in req.params:
             if 'name' in objects[-1]:
                 name = objects[-1]['name']
@@ -191,10 +214,21 @@ class BucketController(Controller):
                 name = quote(name.encode('utf-8'))
             SubElement(elem, 'NextMarker').text = name
         # XXX: really? no NextMarker when no delimiter??
+        SubElement(elem, 'MaxKeys').text = str(tag_max_keys)
+        delimiter = swob.wsgi_to_str(req.params.get('delimiter'))
+        if delimiter:
+            if encoding_type == 'url':
+                delimiter = quote(delimiter)
+            SubElement(elem, 'Delimiter').text = delimiter
+        if encoding_type == 'url':
+            SubElement(elem, 'EncodingType').text = encoding_type
+        SubElement(elem, 'IsTruncated').text = \
+            'true' if is_truncated else 'false'
         return elem
 
-    def _build_list_bucket_result_type_two(self, req, objects, is_truncated):
-        elem = self._build_base_listing_element(req)
+    def _build_list_bucket_result_type_two(self, req, objects, encoding_type,
+                                           tag_max_keys, is_truncated):
+        elem = self._build_base_listing_element(req, encoding_type)
         if is_truncated:
             if 'name' in objects[-1]:
                 SubElement(elem, 'NextContinuationToken').text = \
@@ -204,22 +238,24 @@ class BucketController(Controller):
                     b64encode(objects[-1]['subdir'].encode('utf8'))
         if 'continuation-token' in req.params:
             SubElement(elem, 'ContinuationToken').text = \
-                req.params['continuation-token']
-        if 'start-after' in req.params:
-            SubElement(elem, 'StartAfter').text = \
-                req.params['start-after']
+                swob.wsgi_to_str(req.params['continuation-token'])
+        start_after = swob.wsgi_to_str(req.params.get('start-after'))
+        if start_after is not None:
+            if encoding_type == 'url':
+                start_after = quote(start_after)
+            SubElement(elem, 'StartAfter').text = start_after
         SubElement(elem, 'KeyCount').text = str(len(objects))
-        return elem
-
-    def _finish_result(self, req, elem, tag_max_keys, encoding_type,
-                       is_truncated):
         SubElement(elem, 'MaxKeys').text = str(tag_max_keys)
-        if 'delimiter' in req.params:
-            SubElement(elem, 'Delimiter').text = req.params['delimiter']
+        delimiter = swob.wsgi_to_str(req.params.get('delimiter'))
+        if delimiter:
+            if encoding_type == 'url':
+                delimiter = quote(delimiter)
+            SubElement(elem, 'Delimiter').text = delimiter
         if encoding_type == 'url':
             SubElement(elem, 'EncodingType').text = encoding_type
         SubElement(elem, 'IsTruncated').text = \
             'true' if is_truncated else 'false'
+        return elem
 
     def _add_subdir(self, elem, o, encoding_type):
         common_prefixes = SubElement(elem, 'CommonPrefixes')
@@ -293,11 +329,10 @@ class BucketController(Controller):
         """
         Handle GET Bucket (List Objects) request
         """
-        max_keys = req.get_validated_param(
+        tag_max_keys = req.get_validated_param(
             'max-keys', self.conf.max_bucket_listing)
-        tag_max_keys = max_keys
         # TODO: Separate max_bucket_listing and default_bucket_listing
-        max_keys = min(max_keys, self.conf.max_bucket_listing)
+        max_keys = min(tag_max_keys, self.conf.max_bucket_listing)
 
         encoding_type, query, listing_type, fetch_owner = \
             self._parse_request_options(req, max_keys)
@@ -310,15 +345,12 @@ class BucketController(Controller):
         objects = objects[:max_keys]
 
         if listing_type == 'object-versions':
-            elem = self._build_versions_result(req, objects, is_truncated)
+            func = self._build_versions_result
         elif listing_type == 'version-2':
-            elem = self._build_list_bucket_result_type_two(
-                req, objects, is_truncated)
+            func = self._build_list_bucket_result_type_two
         else:
-            elem = self._build_list_bucket_result_type_one(
-                req, objects, encoding_type, is_truncated)
-        self._finish_result(
-            req, elem, tag_max_keys, encoding_type, is_truncated)
+            func = self._build_list_bucket_result_type_one
+        elem = func(req, objects, encoding_type, tag_max_keys, is_truncated)
         self._add_objects_to_result(
             req, elem, objects, encoding_type, listing_type, fetch_owner)
 
