@@ -176,6 +176,52 @@ class TestS3ApiMultiDelete(S3ApiTestCase):
             ('DELETE', '/v1/AUTH_test/bucket/Key4?multipart-manifest=delete'),
         ])
 
+    @s3acl
+    def test_object_multi_DELETE_with_non_json(self):
+        self.swift.register('DELETE', '/v1/AUTH_test/bucket/Key1',
+                            swob.HTTPNoContent, {}, None)
+        self.swift.register('DELETE', '/v1/AUTH_test/bucket/Key2',
+                            swob.HTTPNotFound, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket/Key3',
+                            swob.HTTPForbidden, {}, None)
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket/Key4',
+                            swob.HTTPOk,
+                            {'x-static-large-object': 'True'},
+                            None)
+        self.swift.register('DELETE', '/v1/AUTH_test/bucket/Key4',
+                            swob.HTTPOk, {}, b'asdf')
+
+        elem = Element('Delete')
+        for key in ['Key1', 'Key2', 'Key3', 'Key4']:
+            obj = SubElement(elem, 'Object')
+            SubElement(obj, 'Key').text = key
+        body = tostring(elem, use_s3ns=False)
+        content_md5 = base64.b64encode(
+            md5(body, usedforsecurity=False).digest()).strip()
+
+        req = Request.blank('/bucket?delete',
+                            environ={'REQUEST_METHOD': 'POST'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Content-Type': 'multipart/form-data',
+                                     'Date': self.get_date_header(),
+                                     'Content-MD5': content_md5},
+                            body=body)
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status.split()[0], '200')
+
+        elem = fromstring(body)
+        self.assertEqual(len(elem.findall('Deleted')), 2)
+        self.assertEqual(len(elem.findall('Error')), 2)
+        self.assertEqual(
+            [tuple(el.find(x).text for x in ('Key', 'Code', 'Message'))
+             for el in elem.findall('Error')],
+            [('Key3', 'AccessDenied', 'Access Denied.'),
+             ('Key4', 'SLODeleteError', 'Unexpected swift response')])
+
+        self.assertEqual(self.s3api.logger.get_lines_for_level('error'), [
+            'Could not parse SLO delete response (200 OK): %s: ' % b'asdf'])
+        self.s3api.logger.clear()
+
     @s3acl
     def test_object_multi_DELETE_quiet(self):
         self.swift.register('DELETE', '/v1/AUTH_test/bucket/Key1',
