@@ -26,7 +26,7 @@ from swift.common.middleware import container_sync
 from swift.proxy.controllers.base import get_cache_key
 from swift.proxy.controllers.info import InfoController
 
-from test.unit import FakeLogger
+from test.debug_logger import debug_logger
 
 
 class FakeApp(object):
@@ -90,7 +90,7 @@ cluster_dfw1 = http://dfw1.host/v1/
     def test_current_invalid(self):
         self.conf = {'swift_dir': self.tempdir, 'current': 'foo'}
         self.sync = container_sync.ContainerSync(self.app, self.conf,
-                                                 logger=FakeLogger())
+                                                 logger=debug_logger())
         self.assertIsNone(self.sync.realm)
         self.assertIsNone(self.sync.cluster)
         info = {}
@@ -136,7 +136,7 @@ cluster_dfw1 = http://dfw1.host/v1/
     def test_missing_from_realms_conf(self):
         self.conf = {'swift_dir': self.tempdir, 'current': 'foo/bar'}
         self.sync = container_sync.ContainerSync(self.app, self.conf,
-                                                 logger=FakeLogger())
+                                                 logger=debug_logger())
         self.assertEqual('FOO', self.sync.realm)
         self.assertEqual('BAR', self.sync.cluster)
         info = {}
@@ -216,6 +216,7 @@ cluster_dfw1 = http://dfw1.host/v1/
         self.assertIn('cs:invalid-sig', req.environ.get('swift.log_info'))
         self.assertNotIn('swift.authorize_override', req.environ)
         self.assertNotIn('swift.slo_override', req.environ)
+        self.assertNotIn('swift.symlink_override', req.environ)
 
     def test_valid_sig(self):
         ts = '1455221706.726999_0123456789abcdef'
@@ -235,6 +236,7 @@ cluster_dfw1 = http://dfw1.host/v1/
         self.assertEqual(ts, resp.headers['X-Timestamp'])
         self.assertIn('swift.authorize_override', req.environ)
         self.assertIn('swift.slo_override', req.environ)
+        self.assertIn('swift.symlink_override', req.environ)
 
     def test_valid_sig2(self):
         sig = self.sync.realms_conf.get_sig(
@@ -250,6 +252,7 @@ cluster_dfw1 = http://dfw1.host/v1/
         self.assertIn('cs:valid', req.environ.get('swift.log_info'))
         self.assertIn('swift.authorize_override', req.environ)
         self.assertIn('swift.slo_override', req.environ)
+        self.assertIn('swift.symlink_override', req.environ)
 
     def test_info(self):
         req = swob.Request.blank('/info')
