@@ -161,6 +161,16 @@ class TestCacheMiddleware(unittest.TestCase):
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 5)
 
+    def test_conf_inline_ratelimiting(self):
+        with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
+            app = memcache.MemcacheMiddleware(
+                FakeApp(),
+                {'error_suppression_limit': '5',
+                 'error_suppression_interval': '2.5'})
+        self.assertEqual(app.memcache._error_limit_count, 5)
+        self.assertEqual(app.memcache._error_limit_time, 2.5)
+        self.assertEqual(app.memcache._error_limit_duration, 2.5)
+
     def test_conf_inline_tls(self):
         fake_context = mock.Mock()
         with mock.patch.object(ssl, 'create_default_context',
@@ -354,6 +364,9 @@ class TestCacheMiddleware(unittest.TestCase):
         # tries is limited to server count
         self.assertEqual(memcache_ring._tries, 4)
         self.assertEqual(memcache_ring._io_timeout, 1.0)
+        self.assertEqual(memcache_ring._error_limit_count, 10)
+        self.assertEqual(memcache_ring._error_limit_time, 60)
+        self.assertEqual(memcache_ring._error_limit_duration, 60)
         self.assertIsInstance(
             list(memcache_ring._client_cache.values())[0]._tls_context,
             ssl.SSLContext)
@@ -384,6 +397,8 @@ class TestCacheMiddleware(unittest.TestCase):
             10.0.0.4:11211
         connect_timeout = 0.5
         io_timeout = 1.0
+        error_suppression_limit = 0
+        error_suppression_interval = 1.5
         """
         memcache_config_path = os.path.join(tempdir, 'memcache.conf')
         with open(memcache_config_path, 'w') as f:
@@ -397,6 +412,9 @@ class TestCacheMiddleware(unittest.TestCase):
         self.assertEqual(memcache_ring._tries, 3)
         # memcache conf options are defaults
         self.assertEqual(memcache_ring._io_timeout, 1.0)
+        self.assertEqual(memcache_ring._error_limit_count, 0)
+        self.assertEqual(memcache_ring._error_limit_time, 1.5)
+        self.assertEqual(memcache_ring._error_limit_duration, 1.5)
 
 
 if __name__ == '__main__':
