@@ -15,20 +15,25 @@
 
 import array
 from contextlib import contextmanager
+import errno
 import json
 import mock
 import os
 from posix import stat_result, statvfs_result
 from shutil import rmtree
 import tempfile
+import time
 import unittest
 from unittest import TestCase
 
 from swift import __version__ as swiftver
 from swift.common import ring, utils
+from swift.common.recon import RECON_RELINKER_FILE, RECON_DRIVE_FILE, \
+    DEFAULT_RECON_CACHE_PATH, server_type_to_recon_file
 from swift.common.swob import Request
 from swift.common.middleware import recon
 from swift.common.storage_policy import StoragePolicy
+from test.debug_logger import debug_logger
 from test.unit import patch_policies
 
 
@@ -160,6 +165,9 @@ class FakeRecon(object):
     def fake_sharding(self):
         return {"sharding_stats": "1"}
 
+    def fake_relinker(self):
+        return {"relinktest": "1"}
+
     def fake_updater(self, recon_type):
         self.fake_updater_rtype = recon_type
         return {'updatertest': "1"}
@@ -205,8 +213,10 @@ class FakeRecon(object):
     def nocontent(self):
         return None
 
-    def raise_IOError(self, *args, **kwargs):
-        raise IOError
+    def raise_IOError(self, errno=None):
+        mock_obj = mock.MagicMock()
+        mock_obj.side_effect = IOError(errno, str(errno))
+        return mock_obj
 
     def raise_ValueError(self, *args, **kwargs):
         raise ValueError
@@ -235,6 +245,7 @@ class TestReconSuccess(TestCase):
         self.real_from_cache = self.app._from_recon_cache
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
         self.frecon = FakeRecon()
+        self.app.logger = debug_logger()
 
         # replace hash md5 implementation of the md5_hash_for_file function
         mock_hash_for_file = mock.patch(
@@ -307,6 +318,11 @@ class TestReconSuccess(TestCase):
             self._create_ring(ringpath, replica_map, self.ring_devs,
                               self.ring_part_shift)
 
+    def _full_recon_path(self, server_type, recon_file=None):
+        if server_type:
+            recon_file = server_type_to_recon_file(server_type)
+        return os.path.join(DEFAULT_RECON_CACHE_PATH, recon_file)
+
     @patch_policies([
         StoragePolicy(0, 'stagecoach'),
         StoragePolicy(1, 'pinto', is_deprecated=True),
@@ -476,11 +492,29 @@ class TestReconSuccess(TestCase):
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
 
     def test_from_recon_cache_ioerror(self):
-        oart = self.frecon.raise_IOError
+        oart = self.frecon.raise_IOError()
         self.app._from_recon_cache = self.real_from_cache
         rv = self.app._from_recon_cache(['testkey1', 'notpresentkey'],
                                         'test.cache', openr=oart)
         self.assertEqual(rv, {'notpresentkey': None, 'testkey1': None})
+        self.assertIn('Error reading recon cache file: ',
+                      self.app.logger.get_lines_for_level('error'))
+        # Now try with ignore_missing but not ENOENT
+        self.app.logger.clear()
+        rv = self.app._from_recon_cache(['testkey1', 'notpresentkey'],
+                                        'test.cache', openr=oart,
+                                        ignore_missing=True)
+        self.assertEqual(rv, {'notpresentkey': None, 'testkey1': None})
+        self.assertIn('Error reading recon cache file: ',
+                      self.app.logger.get_lines_for_level('error'))
+        # Now try again with ignore_missing with ENOENT
+        self.app.logger.clear()
+        oart = self.frecon.raise_IOError(errno.ENOENT)
+        rv = self.app._from_recon_cache(['testkey1', 'notpresentkey'],
+                                        'test.cache', openr=oart,
+                                        ignore_missing=True)
+        self.assertEqual(rv, {'notpresentkey': None, 'testkey1': None})
+        self.assertEqual(self.app.logger.get_lines_for_level('error'), [])
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
 
     def test_from_recon_cache_valueerror(self):
@@ -489,6 +523,8 @@ class TestReconSuccess(TestCase):
         rv = self.app._from_recon_cache(['testkey1', 'notpresentkey'],
                                         'test.cache', openr=oart)
         self.assertEqual(rv, {'notpresentkey': None, 'testkey1': None})
+        self.assertIn('Error parsing recon cache file: ',
+                      self.app.logger.get_lines_for_level('error'))
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
 
     def test_from_recon_cache_exception(self):
@@ -497,6 +533,8 @@ class TestReconSuccess(TestCase):
         rv = self.app._from_recon_cache(['testkey1', 'notpresentkey'],
                                         'test.cache', openr=oart)
         self.assertEqual(rv, {'notpresentkey': None, 'testkey1': None})
+        self.assertIn('Error retrieving recon data: ',
+                      self.app.logger.get_lines_for_level('error'))
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
 
     def test_get_mounted(self):
@@ -646,13 +684,14 @@ class TestReconSuccess(TestCase):
         self.assertEqual(rv, meminfo_resp)
 
     def test_get_async_info(self):
-        from_cache_response = {'async_pending': 5}
+        now = time.time()
+        from_cache_response = {'async_pending': 5, 'async_pending_last': now}
         self.fakecache.fakeout = from_cache_response
         rv = self.app.get_async_info()
         self.assertEqual(self.fakecache.fakeout_calls,
-                         [((['async_pending'],
-                             '/var/cache/swift/object.recon'), {})])
-        self.assertEqual(rv, {'async_pending': 5})
+                         [((['async_pending', 'async_pending_last'],
+                             self._full_recon_path('object')), {})])
+        self.assertEqual(rv, {'async_pending': 5, 'async_pending_last': now})
 
     def test_get_replication_info_account(self):
         from_cache_response = {
@@ -674,7 +713,7 @@ class TestReconSuccess(TestCase):
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['replication_time', 'replication_stats',
                              'replication_last'],
-                             '/var/cache/swift/account.recon'), {})])
+                             self._full_recon_path('account')), {})])
         self.assertEqual(rv, {
             "replication_stats": {
                 "attempted": 1, "diff": 0,
@@ -711,7 +750,8 @@ class TestReconSuccess(TestCase):
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['replication_time', 'replication_stats',
                              'replication_last'],
-                             '/var/cache/swift/container.recon'), {})])
+                             self._full_recon_path('container')),
+                           {})])
         self.assertEqual(rv, {
             "replication_time": 200.0,
             "replication_stats": {
@@ -748,7 +788,7 @@ class TestReconSuccess(TestCase):
                          [((['replication_time', 'replication_stats',
                              'replication_last', 'object_replication_time',
                              'object_replication_last'],
-                             '/var/cache/swift/object.recon'), {})])
+                             self._full_recon_path('object')), {})])
         self.assertEqual(rv, {
             "replication_time": 0.2615511417388916,
             "replication_stats": {
@@ -774,7 +814,7 @@ class TestReconSuccess(TestCase):
         rv = self.app.get_updater_info('container')
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['container_updater_sweep'],
-                            '/var/cache/swift/container.recon'), {})])
+                            self._full_recon_path('container')), {})])
         self.assertEqual(rv, {"container_updater_sweep": 18.476239919662476})
 
     def test_get_updater_info_object(self):
@@ -784,7 +824,7 @@ class TestReconSuccess(TestCase):
         rv = self.app.get_updater_info('object')
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['object_updater_sweep'],
-                            '/var/cache/swift/object.recon'), {})])
+                            self._full_recon_path('object')), {})])
         self.assertEqual(rv, {"object_updater_sweep": 0.79848217964172363})
 
     def test_get_updater_info_unrecognized(self):
@@ -799,7 +839,7 @@ class TestReconSuccess(TestCase):
         rv = self.app.get_expirer_info('object')
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['object_expiration_pass', 'expired_last_pass'],
-                            '/var/cache/swift/object.recon'), {})])
+                            self._full_recon_path('object')), {})])
         self.assertEqual(rv, from_cache_response)
 
     def test_get_auditor_info_account(self):
@@ -815,7 +855,7 @@ class TestReconSuccess(TestCase):
                              'account_auditor_pass_completed',
                              'account_audits_since',
                              'account_audits_failed'],
-                             '/var/cache/swift/account.recon'), {})])
+                             self._full_recon_path('account')), {})])
         self.assertEqual(rv, {"account_auditor_pass_completed": 0.24,
                               "account_audits_failed": 0,
                               "account_audits_passed": 6,
@@ -834,7 +874,7 @@ class TestReconSuccess(TestCase):
                              'container_auditor_pass_completed',
                              'container_audits_since',
                              'container_audits_failed'],
-                             '/var/cache/swift/container.recon'), {})])
+                             self._full_recon_path('container')), {})])
         self.assertEqual(rv, {"container_auditor_pass_completed": 0.24,
                               "container_audits_failed": 0,
                               "container_audits_passed": 6,
@@ -862,7 +902,7 @@ class TestReconSuccess(TestCase):
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['object_auditor_stats_ALL',
                              'object_auditor_stats_ZBF'],
-                             '/var/cache/swift/object.recon'), {})])
+                             self._full_recon_path('object')), {})])
         self.assertEqual(rv, {
             "object_auditor_stats_ALL": {
                 "audit_time": 115.14418768882751,
@@ -909,7 +949,7 @@ class TestReconSuccess(TestCase):
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['object_auditor_stats_ALL',
                              'object_auditor_stats_ZBF'],
-                             '/var/cache/swift/object.recon'), {})])
+                             self._full_recon_path('object')), {})])
         self.assertEqual(rv, {
             "object_auditor_stats_ALL": {
                 'disk1': {
@@ -1098,7 +1138,8 @@ class TestReconSuccess(TestCase):
         rv = self.app.get_driveaudit_error()
         self.assertEqual(self.fakecache.fakeout_calls,
                          [((['drive_audit_errors'],
-                            '/var/cache/swift/drive.recon'), {})])
+                            self._full_recon_path(
+                                None, recon_file=RECON_DRIVE_FILE)), {})])
         self.assertEqual(rv, {'drive_audit_errors': 7})
 
     def test_get_time(self):
@@ -1179,13 +1220,97 @@ class TestReconSuccess(TestCase):
                     }
                 },
             },
+            "sharding_time": 600,
             "sharding_last": 1614136398.6680582}
         self.fakecache.fakeout_calls = []
         self.fakecache.fakeout = from_cache_response
         rv = self.app.get_sharding_info()
+        self.assertEqual(self.fakecache.fakeout_calls, [
+            ((['sharding_stats', 'sharding_time', 'sharding_last'],
+              self._full_recon_path('container')), {})])
+        self.assertEqual(rv, from_cache_response)
+
+    def test_get_relinker_info(self):
+        from_cache_response = {
+            "devices": {
+                "sdb3": {
+                    "parts_done": 523,
+                    "policies": {
+                        "1": {
+                            "next_part_power": 11,
+                            "start_time": 1618998724.845616,
+                            "stats": {
+                                "errors": 0,
+                                "files": 1630,
+                                "hash_dirs": 1630,
+                                "linked": 1630,
+                                "policies": 1,
+                                "removed": 0
+                            },
+                            "timestamp": 1618998730.24672,
+                            "total_parts": 1029,
+                            "total_time": 5.400741815567017
+                        }},
+                    "start_time": 1618998724.845946,
+                    "stats": {
+                        "errors": 0,
+                        "files": 836,
+                        "hash_dirs": 836,
+                        "linked": 836,
+                        "removed": 0
+                    },
+                    "timestamp": 1618998730.24672,
+                    "total_parts": 523,
+                    "total_time": 5.400741815567017
+                },
+                "sdb7": {
+                    "parts_done": 506,
+                    "policies": {
+                        "1": {
+                            "next_part_power": 11,
+                            "part_power": 10,
+                            "parts_done": 506,
+                            "start_time": 1618998724.845616,
+                            "stats": {
+                                "errors": 0,
+                                "files": 794,
+                                "hash_dirs": 794,
+                                "linked": 794,
+                                "removed": 0
+                            },
+                            "step": "relink",
+                            "timestamp": 1618998730.166175,
+                            "total_parts": 506,
+                            "total_time": 5.320528984069824
+                        }
+                    },
+                    "start_time": 1618998724.845616,
+                    "stats": {
+                        "errors": 0,
+                        "files": 794,
+                        "hash_dirs": 794,
+                        "linked": 794,
+                        "removed": 0
+                    },
+                    "timestamp": 1618998730.166175,
+                    "total_parts": 506,
+                    "total_time": 5.320528984069824
+                }
+            },
+            "workers": {
+                "100": {
+                    "drives": ["sda1"],
+                    "return_code": 0,
+                    "timestamp": 1618998730.166175}
+            }}
+        self.fakecache.fakeout_calls = []
+        self.fakecache.fakeout = from_cache_response
+        rv = self.app.get_relinker_info()
         self.assertEqual(self.fakecache.fakeout_calls,
-                         [((['sharding_stats', 'sharding_last'],
-                             '/var/cache/swift/container.recon'), {})])
+                         [((['devices', 'workers'],
+                            self._full_recon_path(
+                                None, recon_file=RECON_RELINKER_FILE)),
+                           {'ignore_missing': True})])
         self.assertEqual(rv, from_cache_response)
 
 
@@ -1221,6 +1346,7 @@ class TestReconMiddleware(unittest.TestCase):
         self.app.get_driveaudit_error = self.frecon.fake_driveaudit
         self.app.get_time = self.frecon.fake_time
         self.app.get_sharding_info = self.frecon.fake_sharding
+        self.app.get_relinker_info = self.frecon.fake_relinker
 
     def test_recon_get_mem(self):
         get_mem_resp = [b'{"memtest": "1"}']
@@ -1496,6 +1622,14 @@ class TestReconMiddleware(unittest.TestCase):
         resp = self.app(req.environ, start_response)
         self.assertEqual(resp, get_sharding_resp)
 
+    def test_recon_get_relink(self):
+        get_recon_resp = [
+            b'{"relinktest": "1"}']
+        req = Request.blank('/recon/relinker',
+                            environ={'REQUEST_METHOD': 'GET'})
+        resp = self.app(req.environ, start_response)
+        self.assertEqual(resp, get_recon_resp)
+
 
 if __name__ == '__main__':
     unittest.main()
