@@ -24,9 +24,9 @@ from gzip import GzipFile
 from tempfile import mkdtemp
 from shutil import rmtree
 from test import listen_zero
+from test.debug_logger import debug_logger
 from test.unit import (
-    make_timestamp_iter, debug_logger, patch_policies, mocked_http_conn,
-    FakeLogger)
+    make_timestamp_iter, patch_policies, mocked_http_conn)
 from time import time
 from distutils.dir_util import mkpath
 
@@ -131,7 +131,7 @@ class TestObjectUpdater(unittest.TestCase):
             'devices': '/some/where/else',
             'mount_check': 'huh?',
             'swift_dir': '/not/here',
-            'interval': '600',
+            'interval': '600.1',
             'concurrency': '2',
             'updater_workers': '3',
             'objects_per_second': '10.5',
@@ -140,7 +140,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(daemon.devices, '/some/where/else')
         self.assertEqual(daemon.mount_check, False)
         self.assertEqual(daemon.swift_dir, '/not/here')
-        self.assertEqual(daemon.interval, 600)
+        self.assertEqual(daemon.interval, 600.1)
         self.assertEqual(daemon.concurrency, 2)
         self.assertEqual(daemon.updater_workers, 3)
         self.assertEqual(daemon.max_objects_per_second, 10.5)
@@ -155,7 +155,6 @@ class TestObjectUpdater(unittest.TestCase):
                 object_updater.ObjectUpdater(conf, logger=self.logger)
 
         check_bad({'interval': 'foo'})
-        check_bad({'interval': '300.0'})
         check_bad({'concurrency': 'bar'})
         check_bad({'concurrency': '1.0'})
         check_bad({'slowdown': 'baz'})
@@ -289,7 +288,7 @@ class TestObjectUpdater(unittest.TestCase):
                 self.stats.successes += 1
                 self.stats.unlinks += 1
 
-        logger = FakeLogger()
+        logger = debug_logger()
         ou = MockObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'false',
@@ -359,7 +358,7 @@ class TestObjectUpdater(unittest.TestCase):
                 self.stats.successes += 1
                 self.stats.unlinks += 1
 
-        logger = FakeLogger()
+        logger = debug_logger()
         ou = MockObjectUpdater({
             'devices': self.devices_dir,
             'mount_check': 'false',
@@ -711,19 +710,24 @@ class TestObjectUpdater(unittest.TestCase):
             'X-Backend-Storage-Policy-Index': str(int(policies[0])),
             'User-Agent': 'object-updater %s' % os.getpid(),
             'X-Backend-Accept-Redirect': 'true',
+            'X-Backend-Accept-Quoted-Location': 'true',
         }
-        # always expect X-Backend-Accept-Redirect to be true
+        # always expect X-Backend-Accept-Redirect and
+        # X-Backend-Accept-Quoted-Location to be true
         do_test(headers_out, expected, container_path='.shards_a/shard_c')
         do_test(headers_out, expected)
 
-        # ...unless X-Backend-Accept-Redirect is already set
+        # ...unless they're already set
         expected['X-Backend-Accept-Redirect'] = 'false'
+        expected['X-Backend-Accept-Quoted-Location'] = 'false'
         headers_out_2 = dict(headers_out)
         headers_out_2['X-Backend-Accept-Redirect'] = 'false'
+        headers_out_2['X-Backend-Accept-Quoted-Location'] = 'false'
         do_test(headers_out_2, expected)
 
         # updater should add policy header if missing
         expected['X-Backend-Accept-Redirect'] = 'true'
+        expected['X-Backend-Accept-Quoted-Location'] = 'true'
         headers_out['X-Backend-Storage-Policy-Index'] = None
         do_test(headers_out, expected)
 
@@ -747,7 +751,8 @@ class TestObjectUpdater(unittest.TestCase):
             'X-Timestamp': timestamp.internal,
             'X-Backend-Storage-Policy-Index': str(int(policy)),
             'User-Agent': 'object-updater %s' % os.getpid(),
-            'X-Backend-Accept-Redirect': 'true'}
+            'X-Backend-Accept-Redirect': 'true',
+            'X-Backend-Accept-Quoted-Location': 'true'}
         for request in requests:
             self.assertEqual('PUT', request['method'])
             self.assertDictEqual(expected_headers, request['headers'])
@@ -954,9 +959,11 @@ class TestObjectUpdater(unittest.TestCase):
             # 1st round of redirects, newest redirect should be chosen
             (301, {'Location': '/.shards_a/c_shard_old/o',
                    'X-Backend-Redirect-Timestamp': ts_redirect_1.internal}),
-            (301, {'Location': '/.shards_a/c_shard_new/o',
+            (301, {'Location': '/.shards_a/c%5Fshard%5Fnew/o',
+                   'X-Backend-Location-Is-Quoted': 'true',
                    'X-Backend-Redirect-Timestamp': ts_redirect_2.internal}),
-            (301, {'Location': '/.shards_a/c_shard_old/o',
+            (301, {'Location': '/.shards_a/c%5Fshard%5Fold/o',
+                   'X-Backend-Location-Is-Quoted': 'true',
                    'X-Backend-Redirect-Timestamp': ts_redirect_1.internal}),
             # 2nd round of redirects
             (301, {'Location': '/.shards_a/c_shard_newer/o',
